<?php

namespace Drupal\currencyapi\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * A class for currency output.
 */
class CurrencyApiController extends ControllerBase {

  /**
   * Fetch data from currencyapi.com.
   */
  public function fetchRates(): array {

    $d = [];
    $d['service'] = \Drupal::service('currencyapi.service');
    $d = $d['service']->fetchExchangeRates();

    return [
      '#markup' => $this->t("Rates updated from <a href='https://currencyapi.com'>currencyapi.com</a>  at %d", ['%d' => $d['meta']['last_updated_at']]),
    ];
  }

  /**
   * Info page.
   */
  public function ratesPage(): array {

    $d = [];
    $d['service'] = \Drupal::service('currencyapi.service');
    $d['currency']['base']['code'] = 'EUR';
    $d['currency']['base']['amount'] = 100;

    $d['all_rates'] = $d['service']->getAllRates();

    foreach ($d['all_rates']['currencyapi'] as $code => $value) {
      if ($value['is_default'] == TRUE) {
        $d['base_currency'] = $code;
      }
    }

    return [
      '#theme' => 'currency_rates',
      '#rates' => $d['all_rates']['currencyapi'],
      '#base_currency' => $d['base_currency'],
      '#attached' => [
        'library' => ['currencyapi/currency_rates'],
      ],
    ];
  }

  /**
   * Json output.
   */
  public function ratesJson($key): JsonResponse {

    // @todo add sopme basic validation on $key.
    $d = [];
    $d['service'] = \Drupal::service('currencyapi.service');
    $d['all_rates'] = $d['service']->getAllRates();

    return new JsonResponse($d['all_rates']);
  }

}
